function [junctions ] = graphIt(BW, I, maxRadius, Amper, Volt, Capac, Electric, Resistor,h)
 
Amper = makeStruct(handleComponnent(Amper, maxRadius), 'Ampermeter');
Volt = makeStruct(handleComponnent(Volt, maxRadius),'Voltmeter');
Volt= removeRedundancyAmper(Amper, Volt, maxRadius);

Capac = makeStruct(handleComponnent(Capac, maxRadius), 'Capac');
Electric = makeStruct(handleComponnent(Electric, maxRadius), 'Electric');
Electric = removeRedundancyElectric(Capac, Electric, maxRadius);

Resistor = makeStruct(handleComponnent(Resistor, maxRadius), 'Resistor');

waitbar(0.80,h,'Finding junctions');  

[beginLines, endLines] = houghToGraph(BW, I);

waitbar(0.90,h,'Graph it!');
junctions = makeList(beginLines,endLines);

junctions = decreaseGap(junctions);

junctions = passOnAllJunctions(junctions, Amper, Volt, Capac, Electric, Resistor , maxRadius);

 waitbar(0.90,h,'Finding junctions');
 
junctions = sameValues(junctions);

 junctions = removeDuplicatesInRow(junctions);

 delete(h);

 plotComponnent(I, Amper, Volt, Capac, Electric, Resistor);
 
 
 
end



function [newJunctions] = removeDuplicatesInRow(junctions)
   
 sizeOfJunctions = size(junctions);
 manyRows = sizeOfJunctions(1);
 newJunctions = [];
 for rows = 1:manyRows
     newCols = 1;
    for cols = 1:sizeOfRow(junctions, rows)
        p = junctions{rows, cols};
        if notBeenHere(newJunctions, rows, p)
            newJunctions{rows, newCols} = p; 
            newCols = newCols + 1;
        end
        
    end
     
 end
     

end

function [bool] = notBeenHere(junctions, rowIndex, p)
   rowsLength =  sizeOfRow(junctions, rowIndex);
    bool = 1;
   for cols = 1:rowsLength
       if isequal(junctions{rowIndex, cols}, p)
           bool = 0;
           break;
       end
   end
end


function [out] = removeRedundancyAmper(mistake, problem, maxRadius)
    out = [];
    sizeElec = size(problem);
    indexOfNew = 1;
    for i= 1:sizeElec(1)
      if   notClosestCompAmper(problem{i}.Voltmeter, mistake, maxRadius);
          out{indexOfNew} = problem{i,1};
     indexOfNew = indexOfNew + 1;
   
      end
    end
end

function [bool] = notClosestCompAmper(point, Capac, maxRadius)
    bool = 1;
    sizeCa = size(Capac);
    for i =1:sizeCa(1)
       if ( pdist([Capac{i}.Ampermeter;point],'euclidean') <= maxRadius) 
            bool = 0;
       end
    end
end


function [out] = removeRedundancyElectric(mistake, problem, maxRadius)
    out = [];
    sizeElec = size(problem);
    indexOfNew = 1;
    for i= 1:sizeElec(1)
      if   notClosestComp(problem{i}.Electric, mistake, maxRadius);
          out{indexOfNew} = problem{i,1};
         indexOfNew =  indexOfNew + 1;
      end
    end
end

function [bool] = notClosestComp(point, Capac, maxRadius)
    bool = 1;
    sizeCa = size(Capac);
    for i =1:sizeCa(1)
       if ( pdist([Capac{i}.Capac;point],'euclidean') <= maxRadius) 
            bool = 0;
       end
    end
end
%passing on all junctions and translate them to components.
function [junctions] = passOnAllJunctions(junctions, Amper, Volt, Capac, Elec, Resistor, maxRadius)
    s = size(junctions);
    
    for i = 1:s(1)
       
        for j = 1:s(2)
        
             junctions{i,j} = getComponent(junctions{i,j} , Amper, Volt, Capac, Elec,Resistor,  maxRadius);
        end
        
    end
    
 
end

function [newJunctions] = sameValues(junctions)
s = size(junctions);
newJunctionIndex = 1;
newJunctions = [];
for i = 1:s(1)
     indexOfComponent = indexOfComponent(newJunctions,junctions{i,1});
     if (indexOfComponent == -1)
         newJunctions = copyRow(newJunctions, junctions, i, newJunctionIndex);
         newJunctionIndex = newJunctionIndex + 1;
     else
         newJunctions = mergeJunctions(newJunctions, junctions, i, indexOfComponent);
       
     end
end
        
end



%get index of the component if Exists
function [index] = indexOfComponent(newJunction, struct)
index = -1;  
s = size(newJunction);
for i = 1:s(1)
    if isequal(newJunction{i,1},  struct)
        index = i;
        return;
    end
    
end

end

%get component around radius.
function [struct] = getComponent(suspicious, Amper, Volt, Capac, Elec, Resistor, radius)
 radius = radius + 10;
struct = suspicious;
for p =1:length(Amper)
    point = Amper{p}.Ampermeter;
    if ( pdist([point;suspicious],'euclidean') <= (radius ))
        struct = Amper{p};
        return;
    end
    
end


for p =1:length(Volt)
    point = Volt{p}.Voltmeter;
        
    if ( pdist([point;suspicious],'euclidean') <= radius)
        struct = Volt{p};
        return;
    end
    
end

for p =1:length(Capac)
    point = Capac{p}.Capac;
    if ( pdist([point;suspicious],'euclidean') <= radius)
        struct = Capac{p};
        return;
    end
    
end

for p =1:length(Elec)
    point = Elec{p}.Electric;
    if ( pdist([point;suspicious],'euclidean') <= radius)
        struct = Elec{p};
        return;
    end
    
end


for p =1:length(Resistor)
    point = Resistor{p}.Resistor;
    if ( pdist([point;suspicious],'euclidean') <= 1.25*radius)
        struct = Resistor{p};
        return;
    end
    
end


end

function [newJunctions] = decreaseGap(junctions)
newJunctions = [];
    newJunctionIndex = 1;
    for index = 1:length(junctions)
    
        point = junctions{index,1};
       
        [leastLength, leastIndex] = leastGapInList(newJunctions, point);
        
        if (leastLength <= 10)
            newJunctions = mergeJunctions(newJunctions, junctions, index, leastIndex);
    
            
        else
            newJunctions = copyRow(newJunctions, junctions, index, newJunctionIndex);
         newJunctionIndex = newJunctionIndex + 1;
        end
        
    end

end


function [number] = sizeOfRow(list, index)
    sizeOfTable = size(list);
    if (index > sizeOfTable(1))
        number = 0;
        return;
    end
    for i =1:sizeOfTable(2)
        if (isempty(list{index,i}))
            number= i -1;
            return;
        end
    end
    number = sizeOfTable(2);
    
    
end
function [newJunctions] = mergeJunctions(newJunctions, junctions, index, leastIndex)
     sizeOfRowNew = sizeOfRow(newJunctions, leastIndex);
     sizeOfRow = sizeOfRow(junctions,index);
    lengthOfCurrentLine = sizeOfRowNew + 1;
    for i = 2:sizeOfRow
        if (notApeerInLine(newJunctions,junctions{index,i}, leastIndex))
           newJunctions{leastIndex,lengthOfCurrentLine} = junctions{index,i};
         lengthOfCurrentLine = lengthOfCurrentLine + 1;
        end
    end
end

function [notAppear] = notApeerInLine(junctions, point, index)
    notAppear = 1;
%     for i = 1:sizeOfRow(junctions,index)
%         passPoint = junctions{index, i};
%         if (pdist([point;passPoint],'euclidean') < 3)
%             notAppear = 0;
%             break;
%         end
%     end
end

function [newJunctions] = copyRow(newJunctions, junctions, index, newJunctionIndex)

for i = 1:sizeOfRow(junctions,index)
    newJunctions{newJunctionIndex,i} = junctions{index,i};
end


end

function [length, index] = leastGapInList(junction, point)

    length = Inf;
    index = 0;
    p = size(junction);
    for i = 1:p(1);
        juncPoint = junction{i,1};
        if (length > ( pdist([point;juncPoint],'euclidean')))
            length = ( pdist([point;juncPoint],'euclidean'));
            index = i;
        end
    end
end

function [listExit] = makeList(beginLines, endLines)

listExit = {};
   for i = 1:length(beginLines)
   
        listExit{2*(i-1)+1,1} = beginLines(i,:);
        listExit{2*(i-1)+1,2} = endLines(i,:);
        listExit{2*i,1} = endLines(i,:);
        listExit{2*i,2} = beginLines(i,:);

   
   end
   
end


function [] = startPath(I, beginLines, endLines, Amper, Volt, Capac)

p = Amper{2};

[point, startPoint, endPoint] = closestPoint(p, beginLines, endLines);
imagesc(I);
title('Path through it.');
hold on;

plotLine(startPoint, endPoint);

[starts, ends] =  findAllLinesFromLine(startPoint, endPoint, beginLines, endLines); 


plotLines(starts, ends);
end


function [point, startPoint, endPoint] = closestPoint(p, beginLines, endLines)

beginLines = cat(1,beginLines, endLines);
closest = Inf;
point =  [];

for index = 1:length(beginLines)
    if (closest > ( pdist([p;beginLines(index,:)],'euclidean')))
        point = beginLines(index,:);
        closest =( pdist([p; beginLines(index,:)],'euclidean'));
        location = index;
    end
end
 
if (location > length(endLines))
    location = location - length(endLines);
end

startPoint = beginLines(location,:);
endPoint = endLines(location,:);
end



function [] = plotComponnent(I, Amper, Volt, Capac, Electric, Resistor)


imshow(I,'InitialMagnification','fit');
title('After Squeeze point');
hold on;

for p =1:length(Amper)
    point = Amper{p}.Ampermeter;
    plot(point(1),point(2), 'r.','MarkerSize',20,'color', [0,0,1])

end


for p =1:length(Volt)
    point = Volt{p}.Voltmeter;
    plot(point(1),point(2),'r.','MarkerSize',20,'color', [0,1,1])

end

for p =1:length(Capac)
    point = Capac{p}.Capac;
    plot(point(1),point(2),'r.','MarkerSize',20,'color', [0,1,0])

end

for p =1:length(Electric)
    point = Electric{p}.Electric;
    plot(point(1),point(2),'r.','MarkerSize',20,'color', [0.1,0.8,0.5])

end

for p =1:length(Resistor)
    point = Resistor{p}.Resistor;
    plot(point(1),point(2),'r.','MarkerSize',20,'color', [0.5,0.8,0.5])

end

end

function [bool] = needToAdd(p1,p2, radius)
    bool = (radius >  (norm(p1-p2)));
end


function [groupComp] = handleComponnent(comp, maxRadius)

groupComp = {};
sizeComp = size(comp);
for runner = 1:sizeComp(1)
 toAdd = 1;
 p = comp(runner,:);
 groupCompSize = size(groupComp);
    for g = 1:groupCompSize(1)
        if (needToAdd(groupComp{g}(1,:) , p,maxRadius))
             groupComp{g} = cat(1, groupComp{g}, p);
             toAdd  = 0;
             continue
        end
        
    end
    if (toAdd)
        groupComp = cat(1, groupComp, {p});
    end
end
groupCompSize = size(groupComp);
for extern = 1:groupCompSize(1)
    s = size(groupComp{extern});
    groupComp{extern} = sum(groupComp{extern},1)./s(1);
    

end

end



function [tempStart, tempEnd] = findAllLinesFromLine(p1, p2, startPoint, endPoint)
    
    m = (p2(2)-p1(2)) / (p2(1)-p1(1));
    eps = 2;
    tempEnd = []; tempStart = []
    
    for i = 1:length(startPoint)
       if  ((m == Inf) &&  inBound(p1, startPoint(i,:), endPoint(i,:)) && abs(startPoint(i,2) - p2(2)) < eps) 
           tempStart = cat(1, tempStart, startPoint(i,:));
           tempEnd = cat(1, tempEnd, endPoint(i,:));
       end
       
       if ((m == 0) && inBound(p1, startPoint(i,:), endPoint(i,:))  && abs(startPoint(i,1) - p2(1)) < eps )
           tempStart = cat(1, tempStart, startPoint(i,:));
           tempEnd = cat(1, tempEnd, endPoint(i,:));
       end
       
       if  ((m == Inf) &&  inBound(p1, startPoint(i,:), endPoint(i,:)) && abs(endPoint(i,2) - p2(2)) < eps  )
           tempStart = cat(1, tempStart, endPoint(i,:));
           tempEnd = cat(1, tempEnd, startPoint(i,:));
       end
       
       if ((m == 0) && inBound(p1, startPoint(i,:), endPoint(i,:)) && abs(endPoint(i,1) - p2(1)) < eps )
           tempStart = cat(1, tempStart, endPoint(i,:));
           tempEnd = cat(1, tempEnd, startPoint(i,:));
       end
    end
    
    Start = [];
    End = [];
    for i = 1:length(tempStart)
        if (p1(1) == tempStart(i,1) && p1(2) == tempStart(i,2) ...
                && p2(1) == tempEnd(i,1) && p2(2) == tempEnd(i,2))
            continue;
        end
      
         if (p2(1) == tempStart(i,1) && p2(2) == tempStart(i,2) ...
                && p1(1) == tempEnd(i,1) && p1(2) == tempEnd(i,2))
           continue;
            
        end
            
            Start = cat(1,Start, tempStart(i,:));
            End =  cat(1,End, tempEnd(i,:));
   
    end
end

function [bool] = inBound(p2, p1, p3)
%    xBound = ((p1(1) <= p3(1)) && (p3(1) <= p2(1))) || ...
%                ((p3(1) <= p1(1)) && (p2(1) <= p3(1)));
%    
%    yBound = ((p1(2) <= p3(2)) && (p3(2) <= p2(2))) || ...
%                ((p3(2) <= p1(2)) && (p2(2) <= p3(2)));
%            
%    bool = xBound && yBound;
%    
          bool = 1;
end

function [] = plotLine(startPoint, endPoint)

          xy = [startPoint(1,:);endPoint(1,:)];
       plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','green');

    
end

function [list] = makeStruct(list, str)


for i = 1:length(list)

    value = list{i};
    list{i} =  struct(str, value);
    
end
end
